// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file VecSymSolver.h */

#ifndef AMDIS_VECSYMSOLVER_H
#define AMDIS_VECSYMSOLVER_H

#include "OEMSolver.h"
#include "MemoryManager.h"
#include "V3Vector.h"

namespace AMDiS {

  // ============================================================================
  // ===== class VecSymSolver ===================================================
  // ============================================================================
  
  /**
   * \ingroup Solver
   * 
   * \brief
   */
  template<typename VectorType>
  class VecSymSolver : public OEMSolver<VectorType>
  {
  public:
    MEMORY_MANAGED(VecSymSolver<VectorType>);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<VectorType>
    {
    public:
      MEMORY_MANAGED(Creator);
      
      virtual ~Creator() {};
      
      /** \brief
       * Returns a new CGSolver object.
       */
      OEMSolver<VectorType>* create() { 
	return NEW VecSymSolver<VectorType>(this->name); 
      };
    };

    /** \brief
     * constructor
     */
    VecSymSolver(::std::string name);
    
    /** \brief
     * destructor
     */
    ~VecSymSolver();
  protected:
    /** \brief
     * Implements OEMSolver<VectorType>::init().
     */
    void init() {
      p = this->vectorCreator->create();
      r = this->vectorCreator->create();
    };
    
    /** \brief
     * Implements OEMSolver<VectorType>::exit().
     */
    void exit() {
      this->vectorCreator->free(p);
    };

    /** \brief
     * Implements OEMSolver<VectorType>::solve().
     */
    int solveSystem(MatVecMultiplier<VectorType> *mv, VectorType *x, VectorType *b);

  private:
    VectorType *r, *p;
  };
}

#include "VecSymSolver.hh"

#endif // AMDIS_VECSYMSOLVER_H

